/*
 * Subset.cpp
 *
 * (C) 2001 MagnumMan-iCoP (James E. King, III)
 */

#include "StdAfx.h"
#include <algorithm>
#include "ShipList.h"

//
// Subset
//
// These routines make a subset out of the current ship list.
// Use these functions to narrow down a selection set.
//
// You can choose to include or exclude the set that you pass
// in the first parameter.  For example:
//
// Subset(std::list< eRace > (kFederation, kLyran))
//		will pare down the list to Federation and Lyran
//
// Subset(std::list< eRace > (kFederation, kLyran), false)
//		will omit Federation and Lyran from the current list
//
// Why use std::list?  Well, because the constants in the API
// are not all bitmaps, and therefore need to be treated either
// like a range (i.e. "kFederation through kLyran") or with
// an inclusion/exclusion list like the one I decided to use.
//

int32 ShipList::Subset(std::list< eRace > &selector, bool include)
{
	SHIP_LIST_ITER(*this)
		
		for (;;)
		{
			bool match = (std::find(selector.begin(), selector.end(), iter->race)) != selector.end();

			if ((match && !include) || (!match && include))
			{
				iter = erase(iter);
				if (iter != end()) continue; else break;
			}

			break;
		}

	SHIP_LIST_ITER_END

	return size();
}

int32 ShipList::Subset(std::list< eHullType > &selector, bool include)
{
	SHIP_LIST_ITER(*this)
		
		for (;;)
		{
			bool match = (std::find(selector.begin(), selector.end(), iter->hullType)) != selector.end();

			if ((match && !include) || (!match && include))
			{
				iter = erase(iter);
				if (iter != end()) continue; else break;
			}

			break;
		}

	SHIP_LIST_ITER_END

	return size();
}

int32 ShipList::Subset(std::list< eClassTypes > &selector, bool include)
{
	SHIP_LIST_ITER(*this)
		
		for (;;)
		{
			bool match = (std::find(selector.begin(), selector.end(), iter->shipClass)) != selector.end();

			if ((match && !include) || (!match && include))
			{
				iter = erase(iter);
				if (iter != end()) continue; else break;
			}

			break;
		}

	SHIP_LIST_ITER_END

	return size();
}

int32 ShipList::Subset(std::list< eTurnMode > &selector, bool include)
{
	SHIP_LIST_ITER(*this)
		
		for (;;)
		{
			bool match = (std::find(selector.begin(), selector.end(), iter->turnMode)) != selector.end();

			if ((match && !include) || (!match && include))
			{
				iter = erase(iter);
				if (iter != end()) continue; else break;
			}

			break;
		}

	SHIP_LIST_ITER_END

	return size();
}

int32 ShipList::Subset(IntegerSubset is, int32 min, int32 max, bool include)
{
	SHIP_LIST_ITER(*this)
		
		for (;;)
		{
			bool match;

			switch (is)
			{
				case kSubsetBPV:		match = ((iter->bpv >= min)            && (iter->bpv <= max));           break;
				case kSubsetSizeClass:	match = ((iter->sizeClass >= min)      && (iter->sizeClass <= max));     break;
				case kSubsetYearAvail:  match = ((iter->yearFirstAvail >= min) && (iter->yearLastAvail <= max)); break;
				default:
					/* notreached */
					match = false;
					break;
			}

			if ((match && !include) || (!match && include))
			{
				iter = erase(iter);
				if (iter != end()) continue; else break;
			}

			break;
		}

	SHIP_LIST_ITER_END

	return size();
}



int32 ShipList::HullClassSubsets(
	ShipList *mon_sl,
	ShipList *bb_sl,
	ShipList *dn_sl,
	ShipList *cv_sl,
	ShipList *bch_sl,
	ShipList *nca_sl,
	ShipList *ca_sl,
	ShipList *cl_sl,
	ShipList *wd_sl,
	ShipList *dd_sl,
	ShipList *ff_sl
	)
{

	SHIP_LIST_ITER(*this)

		Ship s = *iter;

		switch( iter->shipClass )
		{
		case kClassBattleship:
			bb_sl->push_back(*iter);
			break;
		case kClassDreadnought:
			dn_sl->push_back(s);
			break;
		case kClassCarrier:
			cv_sl->push_back(s);
			break;
		case kClassHeavyBattlecruiser:
			bch_sl->push_back(s);
			break;
		case kClassNewHeavyCruiser:
			nca_sl->push_back(s);
			break;
		case kClassHeavyCruiser:
			ca_sl->push_back(s);
			break;
		case kClassLightCruiser:
			cl_sl->push_back(s);
			break;
		case kClassWarDestroyer:
			wd_sl->push_back(s);
			break;
		case kClassDestroyer:
			dd_sl->push_back(s);
			break;
		case kClassFrigate:
			ff_sl->push_back(s);
			break;
		default:
			break;
		}
		if (iter->race == kMonster) {
			mon_sl->push_back(s);
		}	

	SHIP_LIST_ITER_END

	return size();
}
